/*
 * The {@code Array} class implements a generic multidimensional array of
 * objects. Like a primitive array, it contains components that can be
 * accessed using an index to set and get elements from the array. 
 */ 
 
public class Array<E> {

    private int size  = 0;
    private int dim[] = new int[0];
    private int rank  = 0;
    private E   elem  = null;

    private Object data[]  = null;
    
    public Array(final int... dims) {

        this.rank = dims.length;
        this.dim  = new int[rank];
        int size  = 1;

        // compute size of 1-dim internal array
        for (int x = 0; x < dims.length; x++) {
            size   = size * dims[x];
            dim[x] = dims[x];
        }//end for

        this.data = new Object[size];
        this.size = size;

    }// end constructor

    public Array(final Array src) {

        this.rank = src.rank;
        this.dim  = src.dim;
        this.size = src.size;
        this.elem = (E) src.elem;

        this.data = new Object[this.size];

        System.arraycopy(src.data, 0, this.data, 0, this.size);

    }//end constructor copy

    public Array<E> init(final E elem) { 

        this.elem = elem;
        for (int x = 0; x < this.size; x++) {
            this.data[x] = elem;
        } // end for

        return this;

    }// end init

    public Array<E> init(final E[] elem) {

        this.data = elem;
        return this;

    }//end init

    public final Object[] toArray() {
        return this.data;
    }//end toArray

    public int getRank() {
        return this.rank;
    }//end getRank

    public int getDim(final int index) {
        this.isValidIndex(index);
        return this.dim[index];
    }//end getDim

    public int[] getDims(){
    	return this.dim;
    }//end getDims

    public int size() {
        return this.size;
    }//end size

    public E get(final int... idx) {

        return (E) this.data[this.getIndex(idx)];

    }// end get

    public E getAt(final int index) {

        this.isValidIndex(index);
        return (E) this.data[index];

    }// end getAt

    private final int getIndex(final int... idx) {

        isValidDims(idx);

        int index = 0;

        for (int x = 0; x < idx.length; x++) {

            int i = idx[x];

            for (int y = x + 1; y < idx.length; y++) {
                i = i * dim[y];
            } // end for

            index = index + i;
        } // end for

        return index;

    }// end getIndex

    public E set(final E elem, final int... idx) {

        E temp = (E) this.data[this.getIndex(idx)];

        this.data[this.getIndex(idx)] = elem;

        return temp;

    }// end set

    private void isValidDims(final int... idx) {

        if (idx.length != this.dim.length) {
            throw new RuntimeException("Multdimensional Index Rank Error!");
        }//end if

        for (int x = 0; x < idx.length; x++) {

            if (idx[x] >= dim[x]) {
                throw new RuntimeException("Multidimensional Index Overflow Error!");
            }//end if
            
            if (idx[x] < 0) {
                throw new RuntimeException("Multidimensional Index Underflow Error!");
            }//end if

        } // end for

    }// end isValidDims

    private void isValidIndex(final int idx) {

        if (idx < 0) {
            throw new RuntimeException("Linear Index Underflow Error!");
        }//end if

        if (idx > this.size) {
            throw new RuntimeException("Linear Index Overflow Error!");
        }//end if

    }//end isValidIndex

}// end class Array
